datablock PlayerData(PlayerSurvivalArmor : PlayerStandardArmor)
{
  uiName = "Survival Player";
  maxTools = 7;
  jumpForce = 1280;
  isSurvivor = 1;

  //firstPersonOnly = 0;
  //thirdPersonOnly = 0;

  canJet = 0;
  mass = 120;

  //minImpactSpeed = 15;
  //speedDamageScale = 5;
  minImpactSpeed = 16;
  speedDamageScale = 1;

  jumpEnergyDrain = 32;
  minJumpEnergy = 50;

  cameraTilt = 0;
  cameraMaxDist = 4;
  cameraHorizontalOffset = 0.5;
  cameraVerticalOffset = 1;
};

datablock PlayerData(PlayerSurvivalRunningArmor : PlayerSurvivalArmor)
{
  uiName = "";
  maxForwardSpeed = 12; // 10.5

  runForce = 6000;
  jumpForce = 750; // 540

  minRunEnergy = 8; // 2.5
  //minJumpEnergy = 20;

  //runEnergyDrain = 2;
  runEnergyDrain = 1.6; // 2
  //jumpEnergyDrain = 20;
};

function PlayerSurvivalArmor::onAdd(%this, %obj)
{
  Parent::onAdd(%this, %obj);

  %obj.setShapeNameDistance(20);
  %obj.setShapeNameColor("1 1 1");

  %obj.satiation = 0.75 + getRandom() * 0.25;
  %obj.hydration = 0.75 + getRandom() * 0.25;

  %obj.monitorEnergyLevel();
  %obj.tick();
}

function PlayerSurvivalArmor::onTrigger(%this, %obj, %slot, %state)
{
  Parent::onTrigger(%this, %obj, %slot, %state);

  if (%slot == 4 && %state)
  {
    %obj.setDataBlock(PlayerSurvivalRunningArmor);
    %obj.monitorEnergyLevel();
  }
}

function PlayerSurvivalRunningArmor::onTrigger(%this, %obj, %slot, %state)
{
  Parent::onTrigger(%this, %obj, %slot, %state);

  if (%slot == 4 && !%state)
  {
    %obj.setDataBlock(PlayerSurvivalArmor);
    %obj.monitorEnergyLevel();
  }
}

function Player::monitorEnergyLevel(%this, %last)
{
  cancel(%this.monitorEnergyLevel);

  if (%this.getState() $= "Dead" || !isObject(%this.client))
    return;

  %show = %this.getEnergyLevel() < %this.getDataBlock().maxEnergy;

  if (%show != %last)
    commandToClient(%this.client, 'ShowEnergyBar', %show);

  %this.monitorEnergyLevel = %this.schedule(100, monitorEnergyLevel, %show);
}

function Player::tick(%this)
{
  cancel(%this.tick);
  %mount = %this.getObjectMount();

  if (%this.getState() $= "Dead" || (isObject(%mount) && %mount.isCorpseVehicle))
  {
    if (isObject(%this.client))
    {
      commandToClient(%this.client, 'ClearBottomPrint');
      commandToClient(%this.client, 'ClearCenterPrint');
    }

    return;
  }

  //Satiation
  //%drain = 6.25e-005;
  //%drain = 3.25e-005;
  %drain = 4.25e-005;

  if (%this.satiation <= %drain)
    %this.satiation = 0;
  else
    %this.satiation -= %drain;

  if ($Sim::Time - %this.lastSatiationDamage > 0.25 && %this.satiation < 0.02)
  {
    %factor = 1 - (%this.satiation / 0.02);
    %damage = %factor;

    %this.damage(0, 0, %damage, $DamageType::Suicide);
    %this.lastSatiationDamage = $Sim::Time;
  }

  //Hydration
  %drain = 6e-005;

  if (%this.hydration <= %drain)
    %this.hydration = 0;
  else
    %this.hydration -= %drain;

  if ($Sim::Time - %this.lastHydrationDamage > 0.25 && %this.hydration < 0.02)
  {
    %factor = 1 - (%this.hydration / 0.02);
    %damage = %factor;

    %this.damage(0, 0, %damage, $DamageType::Suicide);
    %this.lastHydrationDamage = $Sim::Time;
  }

  //%regen = 0.0025;
  %regen = 0.0035;

  // if (%this.hydration >= 0.5 && %this.satiation >= 0.5 && $Sim::Time - %this.lastDamaged > 30)
  // {
    %this.setDamageLevel(%this.getDamageLevel() - %regen);
  // }

  %factor = 1 - %this.getDamagePercent();
  %text = "<font:palatino linotype:24>\c7Health ";

  if (%factor < 0.2)
    %text = %text @ "\c0";
  else if (%factor < 0.6)
    %text = %text @ "\c3";
  else
    %text = %text @ "\c2";

  %text = %text @ mCeil(%factor * 100) @ "%";

  if (%this.getMountedImage(0).item.reload)
    %text = %text @ "\c7 | Ammo \c6" @ %this.ammoStr;

  %text = %text @ "<just:right>\c7Satiation \c6";
  %text = %text @ mFloatLength(%this.satiation * 100, 0) @ "%";

  %text = %text @ "\c7 | Hydration \c6";
  %text = %text @ mFloatLength(%this.hydration * 100, 0) @ "% \n";

  if (isObject(%mount))
  {
    %speed = vectorLen(setWord(%mount.getVelocity(), 2, 0));
    %mph = (%speed / 2.64) * 6 * 3600 / 5280;

    %text = %text @ "<just:left>\c7Speed \c6";
    %text = %text @ mFloatLength(%mph, 0) @ " MPH";
    %text = %text @ "<just:right>\c7Fuel \c6";
    %text = %text @ mFloatLength(%mount.fuel, 2) @ "G";
    %text = %text @ "\c7 | Integrity \c6";
    %text = %text @ 100 - mFloatLength(%mount.getDamagePercent() * 100, 0);
    %text = %text @ "% \n";
  }

  %item = %this.tool[%this.currTool];

  if (isObject(%item) && %item.itemDesc !$= "")
  {
    %center = "<font:palatino linotype:32>\c3" @ %item.uiName;

    if (%item.invMass !$= "")
      %center = %center @ " [" @ %item.invMass @ "]";

    %center = %center @ "\n\c6" @ %item.itemDesc;
    %center = %center @ "\n\n\n\n";

    if (isObject(%item.image))
      %center = %center @ "\c3LMB - Use";
    else if (%item.satiation > 0 || %item.hydration > 0)
    {
      %center = %center @ "\c3E - ";

      if (%item.satiation < %item.hydration)
        %center = %center @ "Drink";
      else
        %center = %center @ "Eat";
    }
    else
      %center = %center @ "\c0Cannot be used";
  }
  else
  {
    %start = %this.getEyePoint();
    %path = vectorScale(%this.getEyeVector(), 6);
    %end = vectorAdd(%start, %path);

    %mask = (
      $TypeMasks::ItemObjectType |
      $TypeMasks::PlayerObjectType |
      $TypeMasks::VehicleObjectType |
      $TypeMasks::FxBrickObjectType
    );

    %exempt = %this;

    if (%this.isMounted())
      %exempt = %this.getObjectMount();

    %ray = containerRayCast(%start, %end, %mask, %exempt);

    if (%ray)
    {
      if (%ray.getType() & $TypeMasks::ItemObjectType)
      {
        %item = %ray.getDataBlock();
        %center = "<font:palatino linotype:32>\n\n\n\n\n\c3E - Take " @ %item.uiName;

        if (%item.invMass !$= "")
          %center = %center @ " [" @ %item.invMass @ "]";
      }
      else if (%ray.getType() & $TypeMasks::VehicleObjectType)
      {
        %vehicle = %ray.getDataBlock();
        %center = "<font:palatino linotype:32>\n\n\n\n\n\c3E - Enter " @ %vehicle.uiName;
      }
    }
  }

  // %test = %this.clickItemPickup(1);

  // if (%test)
  // {
  //  %item = %test.getDataBlock();
  //  %center = "<font:palatino linotype:32>\n\n\n\n\n";
  //  %center = %center @ "\c3E - Take " @ %item.uiName;

  //  if (%item.invMass !$= "")
  //    %center = %center @ " [" @ %item.invMass @ "]";
  // }

  if (isObject(%this.client))
  {
    %this.client.bottomPrint(%text, 0, 1);
    %this.client.centerPrint(%center, 0);
  }

  %this.tick = %this.schedule(50, tick);
}

function Player::gainSatiation(%this, %amount)
{
  %this.satiation = mClampF(%this.satiation + %amount, 0, 1);
}

function Player::gainHydration(%this, %amount)
{
  %this.hydration = mClampF(%this.hydration + %amount, 0, 1);
}

package PlayerPackage
{
  function serverCmdLight(%client)
  {
    // if (isObject(%client.light) || !isObject(%client.player) ||
    //   !%client.player.getDataBlock().isSurvivor
    // )
    // {
      Parent::serverCmdLight(%client);
    //}
  }

  function Player::playThread(%this, %slot, %sequenceName)
  {
    if (!%this.getDataBlock().isSurvivor || %slot != 3)
    {
      Parent::playThread(%this, %slot, %sequenceName);
      return;
    }

    if (%sequenceName $= "activate")
    {
      %eyePoint = %this.getEyePoint();
      %eyeVector = %this.getEyeVector();

      %ray = containerRayCast(%eyePoint,
        vectorAdd(%eyePoint, vectorScale(%eyeVector, 10)),
        $TypeMasks::All, %this
      );

      if (!%ray || !(%ray.getType() & $TypeMasks::FxBrickObjectType))
        return;
    }

    Parent::playThread(%this, %slot, %sequenceName);

    if (%sequenceName !$= "activate2" || $Sim::Time - %this.lastRapidClick < 0.5)
      return;

    %this.lastRapidClick = $Sim::Time;

    %eyePoint = %this.getEyePoint();
    %eyeVector = %this.getEyeVector();

    %ray = containerRayCast(%eyePoint,
      vectorAdd(%eyePoint, vectorScale(%eyeVector, 5)),
      $TypeMasks::PlayerObjectType |
      $TypeMasks::FxBrickObjectType,
      %this
    );

    if (!%ray)
      return;

    %col = getWord(%ray, 0);

    if (!(%col.getType() & $TypeMasks::PlayerObjectType))
      return;

    %velocity = vectorScale(%this.getForwardVector(), 7.5);
    %velocity = vectorAdd(%velocity, "0 0 5");

    %col.setVelocity(%velocity);
  }

  function GameConnection::spawnPlayer(%this)
  {
    Parent::spawnPlayer(%this);

    for(%i = 4; %i < %this.player.getDatablock().maxTools; %i++)
    {
      messageClient(%this, 'MsgItemPickup', '', %i, 0);
    }
  }

  function Armor::damage(%this, %obj, %src, %pos, %damage, %type)
  {
    Parent::damage(%this, %obj, %src, %pos, %damage, %type);
    %obj.lastDamaged = $Sim::Time;
  }
};

activatePackage("PlayerPackage");
